dloc <- function(x, y, method = 'genetic', start.segs = 3,
                 tol = 0.05, nseeds = 10,
                 mut = 0.1, recomb = 0.5, ext = 0.5){

# x and y give the depths of the datums in two sections. x is
#  the 'best' (i.e. reference) section.
# method, currently only 'genetic'
# start.segs is the number of line segments with which to begin
#  the genetic algorithm
# tol is the tolerance at which to end evolution of the genetic
#  algorithm
# nseeds is the number of random seeds with which to begin
#  the genetic algorithm
# mut is an argument to rnorm giving the sd of the mutation
#  of solutions each generation
# recomb is a value between 0 and 1 giving the amount
#  of recombination among solutions that do not go extinct.
#  1 means random sampling from all living solutions;
#  0 means no recombination.
# ext is the proportion of solutions that go extinct each
#  generation

  if(length(x) == length(y))
    n <- length(x)
  else
    stop('sections to be compared are not of same length')

############################################################
# main

  loc <- matrix(nrow = start.segs + 1, ncol = 2)
  loc[1,] <- c(x[1], y[1])
  loc[nrow(loc),] <- c(x[n], y[n])
  segs <- start.segs
  while(segs == start.segs){
  #while(fitness <= newfit + tol){ # loop through segments >= start.segs
  	cat('----------')
  	cat(segs, 'segments')
  	cat('----------\n')
  	ith <- 1
    while(ith == 1){
    #while(fitness <= newfit + tol){ # loop through generations
  	  xfoo <- function(z){z <- sort(runif(segs, min = min(x),
  	  	                                        max = max(x)))}
      xseeds <- matrix(nrow = segs - 1, ncol = nseeds)
  	  xseeds <- apply(xseeds, 2, xfoo)
  	  yfoo <- function(z){z <- sort(runif(segs, min = min(y),
  	  	                                        max = max(y)))}
  	  yseeds <- matrix(nrow = segs - 1, ncol = nseeds)
  	  yseeds <- apply(yseeds, 2, yfoo)

  	  cat('generation', ith, '\t')
  	
      sse <- getsse(x,y,c(0, xseeds[,1], 20), c(0,yseeds[,1],10))
    
      thissse <- sum(sse$res^2)
      cat('sum of sqared errors', thissse, '\n')
  
  	  if(ith == 1){
  	    plot(x,y)
  	    bestsse <- thissse	
  	  }
  	  
      points(x, sse$fit,
             col = 'red')
      lines(sse$mod,
            col = 'green')
      segments(x, y, x, sse$fit)
        
      if(thissse < bestsse){
        bestsse <- thissse
        bestloc <- thisloc
      }
    
      segs <- segs + 1
    } # end of generation loop
    ith <- ith + 1
  } # end of segments loop
  
  loc <- list(x = xseeds, y = yseeds)

  return(loc)
}

############################################################
# sub
  
getsse <- function(x, y, xrand, yrand){
  res <- rep(0, length(x))
  fit <- rep(0, length(x))
  for (i in 1:length(x)){
    smaller <- max(which(xrand < x[i]))
    larger <- min(which(xrand > x[i]))
    x2 <- xrand[larger]
    x1 <- xrand[smaller]
    y2 <- yrand[larger]
    y1 <- yrand[smaller]
        
    m <- (y2 - y1) / (x2 - x1)
    b <- y1 - m * x1 
    fit[i] <- m * x[i] + b
  }
  mod <- list(x = xrand, y = yrand)
  res <- y - fit
  return(list(mod = mod, fit = fit, res = res))
}

############################################################

#x <- c(0,1,2,3,5,7,9,17,18,20) + rnorm(10)/5
#y <- c(0,1,2,4,4,4,4,8,9,10) + rnorm(10)/5
#plot(x,y)

#x <- c(0.103, 1.336, 2.036, 2.928, 5.123, 6.738,
#       8.998,17.145,17.960,19.753)
#y <- c(0.152, 0.855, 1.784, 4.072, 4.055, 3.992,
#       4.336, 7.963, 9.238,10.162)
#plot(x,y)
#points(x, getsse(x,y,c(0, xseeds[,1], 20),
#       c(0,yseeds[,1],10))$fit,
#       col = 'red')
#lines(getsse(x,y,c(0, xseeds[,1], 20), c(0,yseeds[,1],10))$mod,
#      col = 'green')
       
#dloc(x,y)